import React, { useState, useRef } from 'react';
import { Handle, Position, NodeProps } from 'reactflow';
import { SchemaReactflowRelationWithFunctions } from '../../../model/reactflow';
import { QueryElementTypes } from '@graphpolaris/shared/lib/querybuilder';
import { Popup } from '@graphpolaris/shared/lib/components/Popup';
import { SchemaRelationshipPopup } from './SchemaRelationshipPopup';
import { SchemaEdge } from '../../../model';
import { RelationPill } from '@graphpolaris/shared/lib/components';

import { Tooltip, TooltipContent, TooltipTrigger } from '@graphpolaris/shared/lib/components/tooltip';
import { CardToolTipVis, CardToolTipVisProps } from '@graphpolaris/shared/lib/components/CardToolTipVis';

export const SchemaRelationPill = React.memo(({ id, selected, data, ...props }: NodeProps<SchemaReactflowRelationWithFunctions>) => {
  const [openPopup, setOpenPopup] = useState(false);
  const ref = useRef<HTMLDivElement>(null);

  /**
   * Adds drag functionality in order to be able to drag the relationNode to the schema.
   * @param event React Mouse drag event.
   */
  const onDragStart = (event: React.DragEvent<HTMLDivElement>) => {
    const eventData: SchemaEdge = {
      type: QueryElementTypes.Relation,
      name: id, //TODO id?
      from: data.from,
      to: data.to,
      collection: data.collection,
      label: data.label,
      attributes: Array.from(data.attributes)
        .map((attribute) => attribute.attributes)
        .flat(),
    };
    event.dataTransfer.setData('application/reactflow', JSON.stringify(eventData));
    event.dataTransfer.effectAllowed = 'move';
  };

  /**
   * Displays the NodeQualityPopup when clicked, and hides them when clicked again
   */
  const onClickToggleNodeQualityPopup = (): void => {
    data.toggleNodeQualityPopup(data.collection);
  };

  /**
   * displays the attribute analytics popup menu when clicked, and hides them when clicked again
   */
  const onClickToggleAttributeAnalyticsPopupMenu = (): void => {
    data.toggleAttributeAnalyticsPopupMenu(data.collection);
  };
  return (
    <>
      <div
        className="w-fit h-fit"
        onDragStart={(event) => onDragStart(event)}
        onDragStartCapture={(event) => onDragStart(event)}
        onMouseDownCapture={(event) => {
          if (!event.shiftKey) event.stopPropagation();
        }}
        onClickCapture={(event) => {
          setOpenPopup(!openPopup);
        }}
        draggable
      >
        {openPopup && (
          <Tooltip key={data.name} open={true} boundaryElement={ref} showArrow={true}>
            <TooltipTrigger />
            <TooltipContent side="top">
              <div>
                <CardToolTipVis
                  type="schema"
                  typeOfSchema="relationship"
                  name={data.collection}
                  colorHeader="#0676C1"
                  numberOfElements={1000}
                  connectedFrom={data.from}
                  connectedTo={data.to}
                  data={
                    data.attributes.length > 0
                      ? data.attributes.reduce(
                          (acc, attr) => {
                            if (attr.name && attr.type) {
                              acc[attr.name] = attr.type;
                            }
                            return acc;
                          },
                          {} as Record<string, string>,
                        )
                      : {}
                  }
                />
              </div>
            </TooltipContent>
          </Tooltip>
        )}
        <RelationPill
          draggable
          title={data.collection}
          withHandles="vertical"
          handleUp={
            <Handle
              style={{ pointerEvents: 'none' }}
              id="entitySourceLeft"
              position={Position.Top}
              className={'!rounded-none !bg-transparent !w-full !h-full !border-0 !b-0 !top-0 !bottom-0'}
              type="target"
            ></Handle>
          }
          handleDown={
            <Handle
              style={{ pointerEvents: 'none' }}
              id="entitySourceRight"
              position={Position.Bottom}
              className={'!rounded-none !bg-transparent !w-full !h-full !border-0 !top-0 !bottom-0'}
              type="source"
            ></Handle>
          }
        />
      </div>
    </>
  );
});

SchemaRelationPill.displayName = 'RelationNode';
